/*------------------------------------------------------------------------------

Copyright 2011 University of Aveiro, Portugal, All Rights Reserved.

These programs are supplied free of charge for research purposes only,
and may not be sold or incorporated into any commercial product. There is
ABSOLUTELY NO WARRANTY of any sort, nor any undertaking that they are
fit for ANY PURPOSE WHATSOEVER. Use them at your own risk. If you do
happen to find a bug, or have modifications to suggest, please report
the same to Armando J. Pinho, ap@ua.pt. The copyright notice above
and this statement of conditions must remain an integral part of each
and every copy made of these files.

------------------------------------------------------------------------------*/

#ifndef MODEL_H_INCLUDED
#define MODEL_H_INCLUDED

#define R_MODEL_OFF		0
#define R_MODEL_ON		1

typedef struct
	{
	unsigned	*freqs;
	unsigned	sum;
	}
PModel;

typedef struct
	{
	unsigned	key;	/* The key stored in this entry */
	unsigned	*pos;	/* List of repeating positions */
	unsigned	nPos;
	}
HTableEntry;

typedef struct
	{
	unsigned	size;		/* Size of the hash table */
	unsigned	keySize;	/* Size of the indexing key */
	unsigned	*entrySize;	/* Number of keys in this entry */
	HTableEntry	**entries;	/* The heads of the hash table lists */
	}
HTable;

typedef struct
	{
	unsigned	nSymbols;
	unsigned	pos;		/* Position of the next predicted symbol */
	unsigned	nHits1;		/* Number of times that the model was right */
	unsigned	nHits2;		/* Idem for the case converted base */
	unsigned	nTries;		/* Number of times the model was used */
	double		*probs;		/* Symbol probabilities given by this model */
	}
RModel;


PModel *CreatePModel(unsigned nSymbols);
int StartRModel(RModel *rModel, HTable *hTable, unsigned key, unsigned curPos,
  unsigned useCurPos, unsigned seqSizeAbsDiff);
RModel *CreateRModel(unsigned nSymbols);
HTable *CreateHTable(unsigned keySize, unsigned hSize);
void InsertKeyPos(HTable *hTable, unsigned key, unsigned pos);
HTableEntry *GetHTableEntry(HTable *hTable, unsigned key);
void ComputeRModelProbs(RModel *rModel, Sequence *refSeq, Alphabet *alphabet);
void UpdateRModel(RModel *rModel, Sequence *refSeq, Alphabet *alphabet,
  Symbol symbol);

#endif /* MODEL_H_INCLUDED */

