/*------------------------------------------------------------------------------

Copyright 2011 University of Aveiro, Portugal, All Rights Reserved.

These programs are supplied free of charge for research purposes only,
and may not be sold or incorporated into any commercial product. There is
ABSOLUTELY NO WARRANTY of any sort, nor any undertaking that they are
fit for ANY PURPOSE WHATSOEVER. Use them at your own risk. If you do
happen to find a bug, or have modifications to suggest, please report
the same to Armando J. Pinho, ap@ua.pt. The copyright notice above
and this statement of conditions must remain an integral part of each
and every copy made of these files.

------------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "defs.h"
#include "mem.h"
#include "seqs.h"

/*----------------------------------------------------------------------------*/

Sequence *CreateSequence(unsigned seqSize)

	{
	unsigned n;
	Sequence *seq;

	if(!(seq = (Sequence *)Calloc(1, sizeof(Sequence))))
		{
		fprintf(stderr, "Error: out of memory\n");
		exit(1);
		}

	if(!(seq->bases = (Base *)Calloc(seqSize + SEQUENCE_LEFT_GUARD,
	  sizeof(Base))))
		{
		fprintf(stderr, "Error: out of memory\n");
		exit(1);
		}

	seq->size = seqSize;

	/* This left guard must be set to a constant value */
	for(n = 0 ; n < SEQUENCE_LEFT_GUARD ; n++)
		seq->bases[n] = 'A';

	seq->bases += SEQUENCE_LEFT_GUARD;

	return seq;
	}

/*----------------------------------------------------------------------------*/

Sequence *ReadSequence(char *fName)

	{
	int c, n;
	char str[256];
	FILE *fp;
	Sequence *seq;

	if(!(seq = (Sequence *)Calloc(1, sizeof(Sequence))))
		{
		fprintf(stderr, "Error: out of memory\n");
		exit(1);
		}

	/* Find out if the file can be open for reading */
	if(!(fp = fopen(fName, "r")))
		{
		fprintf(stderr, "Error: unable to open file \"%s\"\n", fName);
		exit(1);
		}

	fclose(fp);

	/* Open the file for possible decompression with gzip */
	sprintf(str, "zcat -f %s", fName);
	if((fp = popen(str, "r")) == NULL)
		{
		fprintf(stderr, "Error: unable to open file \"%s\"\n", fName);
		exit(1);
		}

	if(!(seq->bases = (Base *)Malloc(SEQUENCE_LEFT_GUARD * sizeof(Base))))
		{
		fprintf(stderr, "Error: out of memory\n");
		exit(1);
		}

	/* Read the data file */
	while((c = getc(fp)) != EOF)
		{
		if(seq->size % SEQUENCE_REALLOC_SIZE == 0)
			{
			if(!(seq->bases = (Base *)Realloc(seq->bases,
			  (SEQUENCE_LEFT_GUARD + SEQUENCE_REALLOC_SIZE + seq->size) *
			  sizeof(Base), SEQUENCE_REALLOC_SIZE * sizeof(Base))))
				{
				fprintf(stderr, "Error: out of memory\n");
				exit(1);
				}

			}

		/* Skip fasta description lines */
		if(c == '>' || c == '@')
			{
			while((c = fgetc(fp)) != EOF && c != '\n')
				;

			continue;
			}

		/* Ignore newlines */
		if(c == '\n')
			continue;

		seq->usage[c]++;
		seq->bases[SEQUENCE_LEFT_GUARD + seq->size++] = c;
		}

	/* This left guard must be set to a constant value */
	for(n = 0 ; n < SEQUENCE_LEFT_GUARD ; n++)
		seq->bases[n] = 'A';

	seq->bases += SEQUENCE_LEFT_GUARD;

	fclose(fp);
	return seq;
	}

/*----------------------------------------------------------------------------*/

void AssignHSymbols(Alphabet *alphabet)

	{
	int n;
	
	alphabet->nHSymbols = 5;
	for(n = 0 ; n < 256 ; n++)
		switch(n)
			{
			case 'A':
			case 'a':
				alphabet->hSymbols[n] = 0;
				break;

			case 'C':
			case 'c':
				alphabet->hSymbols[n] = 1;
				break;

			case 'G':
			case 'g':
				alphabet->hSymbols[n] = 2;
				break;

			case 'T':
			case 't':
				alphabet->hSymbols[n] = 3;
				break;

			default:
				alphabet->hSymbols[n] = 4;
				break;

			}

	}

/*----------------------------------------------------------------------------*/

void SetupCaseConv(Alphabet *alphabet)

	{
	int n;

	for(n = 0 ; n < 256 ; n++)
		if(islower(n))
			alphabet->caseConv[n] = toupper(n);

		else
			if(isupper(n))
				alphabet->caseConv[n] = tolower(n);

	}

/*------------------------------------------------------------------------------

	"bases" points to the current symbol

------------------------------------------------------------------------------*/

unsigned GetKey(Base *bases, unsigned keySize, Alphabet *alphabet)

	{
	unsigned key = 0, prod = 1;
	int n;

	for(n = 1 ; n <= keySize ; n++)
		{
		key += alphabet->hSymbols[*(bases - n)] * prod;
		prod *= alphabet->nHSymbols;
		}

	return key;
	}

